/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
package com.ebmwebsourcing.easycommons.stream;

import static org.junit.Assert.*;

import java.io.IOException;
import java.nio.charset.Charset;

import org.junit.Test;

import com.ebmwebsourcing.easycommons.thread.SimultaneousTaskExecutor;

public class DumbExecutorTest {

    private static final String testString = "Hello!\nWorld!";

    private static final String testString2 = "Linux!\nRocks!";

    @Test
    public void testStdOutCapture() {
        testStreamCapture(testString, new StdOutCaptureTester());
    }

    @Test
    public void testStdErrCapture() {
        testStreamCapture(testString, new StdErrCaptureTester());
    }

    private void testStreamCapture(final String testString,
            final StreamCaptureTester streamCaptureTester) {
        DumbExecutor dumbExecutor = new DumbExecutor() {
            public void doRun() {
                streamCaptureTester.printToCapturedStream(testString);
            }
        };
        dumbExecutor.run();
        assertEquals(testString, streamCaptureTester.getCapturedOutput(dumbExecutor));
    }

    @Test
    public void testStdOutCaptureWithUTF16Encoding() {
        testStreamCaptureWithUTF16Encoding(new StdOutCaptureTester());
    }

    @Test
    public void testStdErrCaptureWithUTF16Encoding() {
        testStreamCaptureWithUTF16Encoding(new StdErrCaptureTester());
    }

    public void testStreamCaptureWithUTF16Encoding(final StreamCaptureTester streamCaptureTester) {
        final String testString = new String(new byte[] { 0x00, 0x48, 0x00, 0x45, 0x00, 0x4C, 0x00,
                0x4C, 0x00, 0x4F }, Charset.forName("UTF-16"));
        DumbExecutor dumbExecutor = new DumbExecutor() {
            public void doRun() {
                streamCaptureTester.printToCapturedStream(testString);
            }
        };
        dumbExecutor.run();
        assertEquals("HELLO", streamCaptureTester.getCapturedOutput(dumbExecutor));
    }

    @Test
    public void testStdOutCaptureRollback() {
        testStreamCaptureRollback(new StdOutCaptureTester());
    }

    @Test
    public void testStdErrCaptureRollback() {
        testStreamCaptureRollback(new StdErrCaptureTester());
    }

    private void testStreamCaptureRollback(final StreamCaptureTester streamCaptureTester) {
        DumbExecutor dumbExecutor = new DumbExecutor() {
            public void doRun() {
                DumbExecutor dumbExecutor2 = new DumbExecutor() {
                    public void doRun() {
                        streamCaptureTester.printToCapturedStream(testString2);
                    }
                };
                streamCaptureTester.printToCapturedStream(testString);
                dumbExecutor2.run();
                streamCaptureTester.printToCapturedStream(testString);
            }
        };
        dumbExecutor.run();
        assertEquals(testString + testString, streamCaptureTester.getCapturedOutput(dumbExecutor));
    }

    @Test
    public void testThrowableCapture() {
        final IOException exception = new IOException("test exception");
        DumbExecutor dumbExecutor = new DumbExecutor() {
            public void doRun() throws Throwable {
                throw exception;
            }
        };
        dumbExecutor.run();
        assertEquals(exception, dumbExecutor.getCapturedThrowable());
    }

    @Test
    public void testStdOutAndStdErrAndThrowableCapture() {
        final String traceToOut = "trace to out.";
        final String traceToErr = "trace to err.";
        final IOException exception = new IOException("test exception");
        DumbExecutor dumbExecutor = new DumbExecutor() {
            public void doRun() throws Throwable {
                System.out.print(traceToOut);
                System.err.print(traceToErr);
                throw exception;
            }
        };
        dumbExecutor.run();
        assertEquals(exception, dumbExecutor.getCapturedThrowable());
        assertEquals(traceToOut, dumbExecutor.getCapturedStdOut());
        assertEquals(traceToErr, dumbExecutor.getCapturedStdErr());
    }

    @Test
    public void testSimultaneousStdOutCapture() throws Exception {
        testSimultaneousStreamCapture(new StdOutCaptureTester());
    }

    
    @Test
    public void testSimultaneousStdErrCapture() throws Exception {
        testSimultaneousStreamCapture(new StdErrCaptureTester());
    }
    
    
    public void testSimultaneousStreamCapture(final StreamCaptureTester streamCaptureTester) throws Exception {
        SimultaneousTaskExecutor simultaneousExecutor = new SimultaneousTaskExecutor();
        final int nbThreads = 500;
        for (int i = 0; i < nbThreads; ++i) {
            final String threadTestString = testString + "-" + i;
            // register a dumb executor task...
            simultaneousExecutor.registerTask(new Runnable() {
                @Override
                public void run() {
                    testStreamCapture(threadTestString, streamCaptureTester);
                }
            });
            // but also a task using directly System.err and System.out (which should 
            // not interfere with above tasks...
            simultaneousExecutor.registerTask(new Runnable() {
                @Override
                public void run() {
                    streamCaptureTester.printToCapturedStream("This trace is not captured!\n");
                }
            });
        }
        simultaneousExecutor.executeAllRegisteredTasks();
    }
    
    

    private static abstract class StreamCaptureTester {
        
        abstract String getCapturedOutput(DumbExecutor dumbExecutorUnderTest);

        abstract void printToCapturedStream(String str);

    }

    private static final class StdOutCaptureTester extends StreamCaptureTester {

        @Override
        String getCapturedOutput(DumbExecutor dumbExecutorUnderTest) {
            return dumbExecutorUnderTest.getCapturedStdOut();
        }

        @Override
        void printToCapturedStream(String str) {
            System.out.print(str);
        }
    }

    private static class StdErrCaptureTester extends StreamCaptureTester {

        @Override
        final String getCapturedOutput(DumbExecutor dumbExecutorUnderTest) {
            return dumbExecutorUnderTest.getCapturedStdErr();
        }

        @Override
        void printToCapturedStream(String str) {
            System.err.print(str);
        }
    }

}
