/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
package com.ebmwebsourcing.easycommons.stream;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;

import junit.framework.TestCase;

/**
 * Initial developer(s): Christophe DENEUX
 * 
 * @author cdeneux - Capgemini Sud
 * 
 */
public class ReadInputStreamTest extends TestCase {
    
    private StringBuilder content;

    @Override
    protected void setUp() throws Exception {
        super.setUp();

        this.content = new StringBuilder();
        this.content.append("Hi,\n");
        this.content.append("\n");
        this.content.append("My length is 27 bytes.\n");

    }

    /**
     * Check the processing of
     * <code>{@link ReaderInputStream#available()}</code> after to have read few
     * bytes.
     */
    public void testAvailableAfterLittleRead() {

        try {
            
            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);
            
            final byte[] readBuffer = new byte[3];
            ris.read(readBuffer);
            final int availableBytes = ris.available();
            
            assertTrue(availableBytes > 0);

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of
     * <code>{@link ReaderInputStream#available()}</code> after to have read all
     * bytes.
     */
    public void testAvailableAfterAllRead() {

        try {
            
            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);
            
            final byte[] readBuffer = new byte[1024];
            ris.read(readBuffer);
            final int availableBytes = ris.available();
            
            assertTrue(availableBytes == 0);

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of
     * <code>{@link ReaderInputStream#available()}</code> after to have read all
     * bytes.
     */
    public void testAvailableAfterAllReadPerBlocksLastSmaller() {

        try {

            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);

            this.readPerBlocks(ris, 5);

            final int availableBytes = ris.available();

            assertTrue(availableBytes == 0);

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * after to have read all bytes.
     */
    public void testReadAll() {

        try {
            
            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);
            
            final byte[] readBuffer = new byte[1024];
            final int readBytes = ris.read(readBuffer);
            
            assertEquals(this.content.toString(), new String(readBuffer, 0,
                    readBytes));

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * about reading the reader in two passes.
     */
    public void testReadAllInTwoPasses() {

        try {

            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);
            final StringBuilder result = new StringBuilder();

            byte[] readBuffer = new byte[10];
            int readBytes = ris.read(readBuffer);
            assertEquals(readBuffer.length, readBytes);
            result.append(new String(readBuffer, 0, readBytes));

            readBuffer = new byte[1024];
            readBytes = ris.read(readBuffer);
            result.append(new String(readBuffer, 0, readBytes));

            assertEquals(this.content.toString(), result.toString());

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * about reading the reader byte per byte.
     */
    public void testReadAllByOneByte() {

        final Reader reader = new StringReader(this.content.toString());
        final ReaderInputStream ris = new ReaderInputStream(reader);

        this.readPerBlocks(ris, 1);

    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * about reading the reader bytes blocks per bytes blocks. All blocks have
     * the same size.
     */
    public void testReadAllBySameBytesBlocks() {
        final Reader reader = new StringReader(this.content.toString());
        final ReaderInputStream ris = new ReaderInputStream(reader);

        this.readPerBlocks(ris, 3);
    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * about reading the reader bytes blocks per bytes blocks. The last block is
     * smaller than other ones
     */
    public void testReadAllByBytesBlocksLastSmaller() {
        final Reader reader = new StringReader(this.content.toString());
        final ReaderInputStream ris = new ReaderInputStream(reader);

        this.readPerBlocks(ris, 5);
    }

    private void readPerBlocks(final InputStream ris, final int blockSize) {

        try {

            final StringBuilder result = new StringBuilder();

            byte[] readBuffer = new byte[blockSize];
            int readBytes = 0;
            int charPos = 0; // Use to check that the last itreration returned
            // EOF (-1)
            while (readBytes != -1) {
                readBytes = ris.read(readBuffer);
                assertTrue((charPos >= this.content.length() &&
                                readBytes == -1)
                        || (charPos < this.content.length() &&
                                Math.min(
                                        readBuffer.length,
                                        this.content.length() - result.length()) == readBytes));
                if (readBytes > 0) {
                    result.append(new String(readBuffer, 0, readBytes));
                }
                charPos += blockSize;
            }

            assertEquals(this.content.toString(), result.toString());

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

}
